objects = {}
function newObject(id, base, params)
	if objects[id] then 
		print("there is already an object with id", id)
		return
	end
	
	local o = {
		_id = id,
		_dead = false, 
	}
	
	if base then
		setmetatable(o, base)
	end
	
	for k,v in pairs(params) do
		o[k] = v
	end
	
	objects[id] = o
end

function signal(id, name, ...)
	local o = objects[id]
	local label = "on_" .. name
	if o then
		if o._dead then
			return true
		end
	
		local f = o[label]
		if f and type(f) == "function" then
			f(o, ...)
		else
			local h = rawget(getmetatable(o) or {}, label)
			if h then
				h(o, ...)
			else
				return true
			end
		end
	else
		return true
	end
end


function bindEvent(event, id, name, fun)
	registerEvent(event, function(...)
		if fun then
			return signal(id, name, unpack(fun(...)))
		else
			return signal(id, name, ...)
		end
	end)
end

function bindGuardedEvent(event, guard, id, name, fun)
	registerEvent(event, function(...)
		if guard(...) then
			if fun then
				return signal(id, name, unpack(fun(...)))
			else
				return signal(id, name, ...)
			end
		end
	end)
end

function copy(src, init)
	local o = {}
	for k,v in pairs(src) do o[k] = v end
	if type(init) == "table" then
		for k,v in pairs(init) do o[k] = v end
	end
	return o
end
