love.filesystem.load( "lib.vardump.lua" )()
love.filesystem.load( "lib.oop.lua" )()
love.filesystem.load( "lib.events.lua" )()
love.filesystem.load( "lib.objects.lua" )()
love.filesystem.load( "lib.cam.lua" )()
love.filesystem.load( "lib.draw.lua" )()
love.filesystem.load( "lib.game.lua" )()
love.filesystem.load( "lib.util.lua" )()
love.filesystem.load( "lib.job.lua" )()

love.filesystem.load( "animation.lua" )()

love.filesystem.load( "media.lua" )()
love.filesystem.load( "types.lua" )()

love.filesystem.load( "obj.timer.lua" )()
love.filesystem.load( "obj.level.lua" )()

gPaused = false
kState_Title 			= 1
kState_Menu 			= 2
kState_Credits 			= 3
kState_Dead 			= 4
kState_Finished 		= 5
kState_Game 			= 6
gState = 0

gState_Draw = {}
gState_Update = {}
gState_Enter = {}
gState_Leave = {}
gState_KeyPressed = {}

gCurrentLevel = 1
gLevelFiles = {
	"test.txt",
	"1.txt",
	"2.txt",
	"3.txt",
	"4.txt",
	"5.txt",
	"6.txt",
	"7.txt",
	"8.txt",
	"9.txt",
}

function switchState(state)
	if state == gState then return end
	
	if gState_Leave[gState] then gState_Leave[gState]() end
	gState = state
	if gState_Enter[gState] then gState_Enter[gState]() end
end

function love.load()
	love.filesystem.setIdentity( "boelderdaesch" )
	
	gTimerFall = Timer{250; tick = function(self) calculateTickFall() end}
	gTimerSlime = Timer{2000; tick = function(self) calculateTickSlime() end}
	gTimerMonster = Timer{500; tick = function(self) calculateTickMonster() end}
	gTimerGamePlay = Timer{250; tick = function(self) calculateTickGamePlay() end}
	
	switchState(kState_Title)
end


function love.update(dt)
	job.step()
	if gState_Update[gState] then gState_Update[gState](dt) end
end

--------------- title ---------------------------
gState_Enter[kState_Title] = function()
	invokeLater(1000, function() switchState(kState_Menu) end)
end

gState_Draw[kState_Title] = function()
	love.graphics.draw(gImage_Title, 0,0)
end

--------------- credits ---------------------------
gState_KeyPressed[kState_Credits] = function(key, unicode)
	switchState(kState_Menu)
end

gState_Draw[kState_Credits] = function()
	love.graphics.draw(gImage_Credits, 0,0)
end

--------------- dead ---------------------------
gState_KeyPressed[kState_Dead] = function(key, unicode)
	switchState(kState_Menu)
end

gState_Draw[kState_Dead] = function()
	love.graphics.draw(gImage_GameOver, 0,0)
end

--------------- finish ---------------------------
gState_KeyPressed[kState_Finished] = function(key, unicode)
	switchState(kState_Menu)
end

gState_Draw[kState_Finished] = function()
	love.graphics.draw(gImage_Finished, 0,0)
end

--------------- menu ---------------------------
gMenuCurrentSelection = 1

gState_Enter[kState_Menu] = function()
end

gState_Draw[kState_Menu] = function()
	love.graphics.draw(gImage_Menu, 0,0)
	
	local x,y = 200,200
	local dx,dy = 0,78
	
	for i = 1,3 do
		if i == gMenuCurrentSelection then
			love.graphics.draw(gImage_MenuItems_On[i], x,y)
		else
			love.graphics.draw(gImage_MenuItems_Off[i], x,y)
		end
		
		x = x + dx
		y = y + dy
	end
end

gState_KeyPressed[kState_Menu] = function(key, unicode)
	if key == "escape" then os.exit() end
	
	if key == "up" then gMenuCurrentSelection = gMenuCurrentSelection - 1 end
	if key == "down" then gMenuCurrentSelection = gMenuCurrentSelection + 1 end
	gMenuCurrentSelection = Clamp(gMenuCurrentSelection, 1,3)
	
	if key == "return" then
		if gMenuCurrentSelection == 1 then
			-- new game
			switchState(kState_Game)
		elseif gMenuCurrentSelection == 2 then
			-- credits
			switchState(kState_Credits)
		elseif gMenuCurrentSelection == 3 then
			-- exit
			os.exit()
		end
	end
end

---------------- game -----------------------------
gState_KeyPressed[kState_Game] = function(key, unicode)
	local sf = 1.5
	if key == "q" then gCameraS = gCameraS * sf end
	if key == "e" then gCameraS = gCameraS / sf end
	if key == "1" then gCameraS = gCameraSStart end
	if key == "2" then gCameraS = 1 end

	if key == "r" then 
		playerDie()
	end
	
	if key == "z" then gPaused = not gPaused end
	if key == "u" then levelFinished() end
	if key == "i" then gPlayerLives = gPlayerLives + 1 end

	if key == "up" then movePlayer(0, -1) end
	if key == "down" then movePlayer(0, 1) end
	if key == "left" then movePlayer(-1, 0) end
	if key == "right" then movePlayer(1, 0) end
	
	if key == "escape" then switchState(kState_Menu) end
end

gState_Enter[kState_Game] = function()
	loadLevel(gLevelFiles[gCurrentLevel])
end

gState_Update[kState_Game] = function(dt)
	--~ local tf = 1000
	--~ if love.keyboard.isDown( "w" ) then gCameraY = gCameraY - tf * dt end
	--~ if love.keyboard.isDown( "s" ) then gCameraY = gCameraY + tf * dt end
	--~ if love.keyboard.isDown( "a" ) then gCameraX = gCameraX - tf * dt end
	--~ if love.keyboard.isDown( "d" ) then gCameraX = gCameraX + tf * dt end
	
	-- camera movement
	local px,py = tile2WorldPosition(gPlayerX, gPlayerY)
	ensurePointIsVisible(px,py, 100)
	
	gTimerFall:update(dt)
	gTimerSlime:update(dt)
	gTimerMonster:update(dt)
	gTimerGamePlay:update(dt)


	-- update all animations
	for k,v in pairs(gTypes) do
		if v.anim then
			v.anim:update(dt)
		end
	end
end

gState_Draw[kState_Game] = function()
	gLevel:draw(0, 0)
	love.graphics.print("FPS " .. love.timer.getFPS(), 750, 16)
	drawHud()
end

----------------------------------------------------------

function movePlayer(dx,dy)
	if gPaused then return end

	local oldx, oldy = gPlayerX, gPlayerY
	local newx, newy = oldx + dx, oldy + dy
	
	local tile = gLevel:getTile(newx,newy)
	
	if tile.deadly then
		gLevel:moveTile(oldx, oldy, dx, dy)
		playerDie()
	else
	
		if tile.movable and dy == 0 then
			-- move stuff around
			local nextTile = gLevel:getTile(newx+dx,newy+dy)
			
			if nextTile.empty then
				love.audio.play(gSound_Move)
			
				gLevel:moveTile(newx, newy, dx, dy)

				gLevel:moveTile(oldx, oldy, dx, dy)
				gPlayerX, gPlayerY = newx, newy
				
				if tile.on_player_enter then
					tile.on_player_enter(newx,newy)
				end
			end
		elseif not tile.solid then
			-- normal blocked movement
			gLevel:moveTile(oldx, oldy, dx, dy)
			gPlayerX, gPlayerY = newx, newy
			
			if tile.on_player_enter then
				tile.on_player_enter(newx,newy)
			end
		end
		
	end
end

function calculateTickSlime()
	if gPaused then return end
	
	local slime = gTypes["Slime"]
	local count = 0

	-- slime
	gLevel:applyRule(
		function(x,y) return gLevel:getTile(x,y) == slime and gLevel:getTile(x+1,y).empty end, 
		function(x,y) count = count + 1 gLevel:moveTile(x,y,1,0) end
	)
	gLevel:applyRule(
		function(x,y) return gLevel:getTile(x,y) == slime and gLevel:getTile(x-1,y).empty end, 
		function(x,y) count = count + 1 gLevel:moveTile(x,y,-1,0) end
	)
	gLevel:applyRule(
		function(x,y) return gLevel:getTile(x,y) == slime and gLevel:getTile(x,y+1).empty end, 
		function(x,y) count = count + 1 gLevel:moveTile(x,y,0,1) end
	)
	gLevel:applyRule(
		function(x,y) return gLevel:getTile(x,y) == slime and gLevel:getTile(x,y-1).empty end, 
		function(x,y) count = count + 1 gLevel:moveTile(x,y,0,-1) end
	)
	
	if count > 0 then
		love.audio.play(gSound_Slime)
	end
end


function Client_GetTicks()
	return love.timer.getTime() * 1000
end

function levelFinished()
	job.create(function()
		gPaused = true
		
		love.audio.play(gSound_LevelFinished)
		
		job.wait(2000)

		gCurrentLevel = gCurrentLevel + 1
		
		if gLevelFiles[gCurrentLevel] then
			loadLevel(gLevelFiles[gCurrentLevel])
		else
			switchState(kState_Finished)
		end

		gPaused = false
	end)
end

function playerDie()
	love.audio.play(gSound_CrashHuman)

	gPlayerLives = gPlayerLives - 1
	
	gLevel:replace(gTypes["Player"], gTypes["Grave"])
	
	if gPlayerLives == 0 then
		switchState(kState_Dead)
	else
		job.create(function()
			gPaused = true
			job.wait(2000)
			restartLevel()
			gPaused = false
		end)
	end
end

function calculateTickMonster()
	if gPaused then return end
	
	local monster = gTypes["Monster"]
	local player = gTypes["Player"]

	local dirs = { {1,0}, {-1,0}, {0,1}, {0,-1} }
	
	gLevel:applyRule(
		function(x,y) return gLevel:getTile(x,y) == monster end, 
		function(x,y) 
			-- chose empty directions
			local mydirs = {}
			for k,v in pairs(dirs) do
				local dx, dy = unpack(v)
				local tile = gLevel:getTile(x+dx,y+dy)
				if tile == player or tile.empty then
					table.insert(mydirs, v)
				end
			end
		
			local n = table.getn(mydirs)
			if n > 0 then
				local dx, dy = unpack(mydirs[math.random(1,n)])
				local tile = gLevel:getTile(x+dx,y+dy)
				
				if tile == player then
					playerDie()
				else
					gLevel:moveTile(x,y,dx,dy)
				end
			end
		end
	)
end

function calculateTickGamePlay()
	if gPaused then return end
	
	local diamonds = gLevel:countType(gTypes["Diamond"])
	local closedDoors = gLevel:countType(gTypes["DoorClosed"])
	
	if diamonds == 0 and closedDoors > 0 then
		-- open closed doors
		gLevel:replace(gTypes["DoorClosed"], gTypes["DoorOpen"])
		love.audio.play(gSound_AllCollected)
	elseif diamonds > 0 then
		-- close opened doors
		gLevel:replace(gTypes["DoorOpen"], gTypes["DoorClosed"])
	end
	
	gDiamondsToCollect = diamonds
end

function calculateTickFall()
	if gPaused then return end

	local empty = gTypes["Empty"]
	
	-- headcrashes
	gLevel:applyRule(
		function(x,y) 
			if not gLevel:getTile(x,y).pinned then
				local speed = gLevel:getCustomParam(x,y) or 0
				return speed >= 1
			end
		end, 
		function(x,y) 
			local nextTile = gLevel:getTile(x,y+1)
			local curTile = gLevel:getTile(x,y)
			
			if nextTile.breakable and nextTile.on_headcrash then
				nextTile.on_headcrash(x,y+1)
			elseif not nextTile.empty and curTile.on_hit_ground then
				curTile.on_hit_ground(x,y)
			end
			gLevel:setCustomParam(x,y,0)
		end
	)
		
	-- fall down
	gLevel:applyRule(
		function(x,y) 
			if not gLevel:getTile(x,y).pinned then
				local speed = gLevel:getCustomParam(x,y) or 0
				local falling = gLevel:getTile(x,y+1).empty

				-- update falling speed
				if falling then
					gLevel:setCustomParam(x,y,speed + 1)
				else
					gLevel:setCustomParam(x,y,0)
				end
				
				return falling
			else
				return false
			end
		end, 
		function(x,y) 
			gLevel:moveTile(x,y,0,1) 
		end
	)
	
	-- fall to the side
	gLevel:applyRule(
		function(x,y) return 
			not gLevel:getTile(x,y).pinned and 
			gLevel:getTile(x+1,y).empty and
			gLevel:getTile(x+1,y+1).empty and
			not gLevel:getTile(x,y+1).empty
		end, 
		function(x,y) gLevel:moveTile(x,y,1,0) end
	)
	gLevel:applyRule(
		function(x,y) return 
			not gLevel:getTile(x,y).pinned and 
			gLevel:getTile(x-1,y).empty and
			gLevel:getTile(x-1,y+1).empty and
			not gLevel:getTile(x,y+1).empty
		end, 
		function(x,y) gLevel:moveTile(x,y,-1,0) end
	)
	
end

function love.keypressed(key, unicode)
	--~ print(key, unicode)
	if gState_KeyPressed[gState] then gState_KeyPressed[gState](key, unicode) end
end

function drawHud()
	local gfx = love.graphics.newImage(gGfxPath .. "herz.png")
	local x = 3
	local y = 3
	local dx = 23
	
	for i = 1,gPlayerLives do
		love.graphics.draw(gfx, x, y, 0, 0.25, 0.25) 
		x = x + dx
	end
	
	-- remaining diamonds
	gfx = love.graphics.newImage(gGfxPath .. "diamant.png")
	love.graphics.draw(gfx, 300, 3, 0, 0.25, 0.25) 
	love.graphics.print(gDiamondsToCollect or 0, 330, 20)
	
	-- logo
	gfx = love.graphics.newImage(gGfxPath .. "minilogo.png")
	love.graphics.draw(gfx, 5, 600 - 2 - 36, 0, 1, 1) 
end

function love.draw()
	if gState_Draw[gState] then gState_Draw[gState]() end
end
