
Level = Object{
	_init = {"filename"},
	
	_construct = function(self)
		self:loadLevelFile()
		self:prepareLevelData()
		self.customParam = {}
	end,
	
	draw = function(self,dx,dy)
		dx = dx or 0
		dy = dy or 0
		
		self:visitLevel(function(x,y,content)
			local px = x * gTileW
			local py = y * gTileH
		
			if content.transparent then 
				drawImage(gBackgroundGfx, dx + px, dy + py)
			end

			if content.anim then
				drawAnim(content.anim, dx + px, dy + py)
			elseif content.gfx then
				drawImage(content.gfx, dx + px, dy + py)
			end
		end)
	end,
	
	isSolid = function(self,tileX,tileY)
		return self.data[tileX][tileY].solid
	end,

	isEmpty = function(self,tileX,tileY)
		return self.data[tileX][tileY].empty
	end,
	
	setTile = function(self,tileX,tileY,typ)
		self.data[tileX] = self.data[tileX] or {}
		self.data[tileX][tileY] = typ
	end,
	
	getTile = function(self,tileX,tileY)
		local t = self.data[tileX]
		if t then
			return self.data[tileX][tileY] or gTypes["Empty"]
		else
			return gTypes["Empty"]
		end
	end,
	
	setCustomParam = function(self,tileX,tileY,param)
		self.customParam[tileX] = self.customParam[tileX] or {}
		self.customParam[tileX][tileY] = param
	end,
	
	getCustomParam = function(self,tileX,tileY)
		local t = self.customParam[tileX]
		if t then
			return self.customParam[tileX][tileY] or nil
		else
			return nil
		end
	end,
	
	
-- -----------------------
	
	-- see visitLevelString
	visitLevel = function(self, fun)
		for x, yaxis in pairs(self.data) do
			for y, content in pairs(yaxis) do
				fun(x,y,content)
			end
		end		
	end,

	-- callback fun : function f(x,y,letter)
	-- the callback can return true to cancel the iteration
	visitLevelString = function(self, fun)
		local x = 0
		local y = 0
		local len = string.len(self.levelString)
		local pos = 0
		
		while pos < len do
			local letter = string.sub(self.levelString, pos + 1, pos + 1)
			
			-- stop if callback returns true
			if fun(x,y,letter) then
				return
			end
			
			pos = pos + 1
			x = x + 1

			if pos > 0 and math.mod(pos, self.maxWidth) == 0 then
				x = 0
				y = y + 1
			end
		end		
	end,
	
	moveTile = function(self,x,y,dx,dy)
		local tile = self:getTile(x,y)
		local param = self:getCustomParam(x,y)
		
		self:setTile(x,y,letterToType(tile.remainings) or gTypes["Empty"])
		self:setCustomParam(x,y,nil)
		
		self:setTile(x+dx,y+dy,tile)
		self:setCustomParam(x+dx,y+dy,param)

		return tile
	end,

	findStartPos = function(self)
		local startX, startY = 0,0

		self:visitLevelString(function(x,y,letter)
			if letter == "S" then
				startX, startY = x, y
				return true
			end
		end)	
	
		return startX, startY
	end,

	prepareLevelData = function(self)
		local minx, miny = 10000, 10000
		local maxx, maxy = -10000, -10000
		
		local level = {}
		
		self:visitLevelString(function(x,y,letter)
			level[x] = level[x] or {}
			level[x][y] = letterToType(letter)
			
			minx = math.min(minx, x)
			miny = math.min(miny, y)
			maxx = math.max(maxx, x)
			maxy = math.max(maxy, y)
		end)

		self.data = level
		
		self.minx = minx
		self.miny = miny
		self.maxx = maxx
		self.maxy = maxy
	end,

	loadLevelFile = function(self)
		local it = love.filesystem.lines("level/" .. self.filename)
		local maxLen = 0
		
		for line in it do
			maxLen = math.max(maxLen, string.len(line))
		end

		local s = ""
		it = love.filesystem.lines("level/" .. self.filename)
		for line in it do
			-- concat space padded lines
			s = s .. line .. string.rep(" ", maxLen - math.max(0, string.len(line)))
		end
		
		self.levelString = s
		self.maxWidth = maxLen
	end,
	
	copyData = function(self)
		local data = {}
		
		for x,yaxis in pairs(self.data) do
			for y,content in pairs(yaxis) do
				data[x] = data[x] or {}
				data[x][y] = content
			end
		end
		
		return data
	end,
	
	applyRule = function(self, condition, transition) 
		local oldData = self.data
		local newData = self:copyData()
		
		for x = self.minx, self.maxx do
			for y = self.miny, self.maxy do
				self.data = oldData
				if condition(x,y) then
					self.data = newData
					transition(x,y)
				end
			end
		end
		
		self.data = newData
	end,
	
	replace = function(self, search, replace)
		self:visitLevel(function(x,y,content)
			if content == search then
				self:setTile(x,y,replace)
			end
		end)
	end,
	
	countType = function(self, typ)
		local count = 0
		
		self:visitLevel(function(x,y,content)
			if content == typ then
				count = count + 1
			end
		end)
		
		return count
	end,
}
